function [Homos, ID_valid]=pick_points_homos(path_base, path_dataset, img_format)
%interface for user to pick corresponding points between the base image and
%each image in the database.

%calculate homography mappings: Homo( imgbase to img2)
%my_displaymatch.m homo_ransac.m


%Options and thresholds
Option_ransac=false;
Option_tform = true;
flag_second_scan = false;
thresh_max_error=3.5;
num_point_pairs_base = 30;
num_point_pairs = 20;
%if small, the mean error will drop (some deforming point matches are treated as mismatches)
%human observance: guarantee that the outliers are either mismatches or
%deforming points
%-------------------------------------------------------------------------
%processing the base image
imgbase=imread(path_base);
I1 = single(rgb2gray(imgbase));
%this is to cut non-planar part only used for pelvis dataset
%choose not to re-select the points, so keep this part
if strcmp(path_dataset(end-4:end), 'clip3')
    imgbase=imgbase(:, 130:1100, :);
    I1=I1(:, 130:1100, :);
end
name_points1=[path_dataset '\points\points_base_all.txt'];

figure(1);
imshow(imgbase);
if ~exist(name_points1, 'file')
    hold on;
    %pick points in base-----------------
    for i=1:num_point_pairs_base
        [x1,y1]=ginput(1);
        pt1_all(i,:)=[x1,y1];
        plot(x1,y1,'*g');
    end
    hold off;
    dlmwrite(name_points1,pt1_all);%save all points in base
else
    pt1_all=dlmread(name_points1);
end

hold on;
plot(pt1_all(:,1),pt1_all(:,2),'or');

%obtain the paths for all other images
str_format = ['img*.' img_format];
struct_dir = dir(fullfile(path_dataset, str_format));
num_img=length(struct_dir);
img_name_all=cell(num_img,1);
for i=1:num_img
    img_name_all{i}=struct_dir(i).name;
end
ID_valid = false(num_img,1);
Homos = zeros(num_img*3,3);%each image has one homography matrix (3*3), stores in 3*(ID-1)+1:3*ID
err_mean_record = zeros(num_img,1)-1;
var_record = zeros(num_img,1)-1;
err_mean_test_cv_record = zeros(num_img,1)-1;
var_test_cv_record = zeros(num_img,1)-1;
err_mean_train_cv_record = zeros(num_img,1)-1;
var_train_cv_record = zeros(num_img,1)-1;
%calculate the homo between the base image and all the other images
ID=1;
while ID <=3%num_img
    pt2=[];
    %read in each image
    name_img2 = img_name_all{ID};
    fprintf('Image name: %s\n', name_img2);
	img2 = imread([path_dataset '\' name_img2]);
    name_points2=[path_dataset '\points\points_' name_img2(end-8:end-4) '.txt'];
    if ~exist(name_points2, 'file')
    %pick points in I2------------------------
        num_valid=0;
        m=1;
        while m<=num_point_pairs_base && num_valid<num_point_pairs
            figure(1);
            imshow(imgbase);
            hold on;
            plot(pt1_all(m,1),pt1_all(m,2),'*g');
            hold off;
            figure(2);
            imshow(img2);
            if size(pt2,1)>1
                hold on;
                plot(pt2(:,1),pt2(:,2),'*b');
                hold off;
            end
            [x2,y2,btn]=ginput(1);
            if btn == 'r'%r: repeat this point
                num_valid = num_valid-1;%repeat last point
                m=m-1;
            else
                if btn ~= 's' && x2>10%s: skip this point %if click out of the range, skip this point
                    num_valid = num_valid+1;%this point is valid
                    pt1(num_valid,1)=pt1_all(m,1);
                    pt1(num_valid,2)=pt1_all(m,2);
                    pt2(num_valid,1)=x2;
                    pt2(num_valid,2)=y2;
                    hold on;
                    plot(x2,y2,'*g');
                    hold off;
                end
                m=m+1;
            end
        end
        if num_valid<num_point_pairs
            fprintf('-------------------Not enough coverage. Image ID: %d is invalid--------------------------\n', ID);
            continue;
        end
        pt_pairs(:,1:2)=pt2;
        pt_pairs(:,3:4)=pt1;%first two colum img2, last two base
        dlmwrite(name_points2,pt_pairs);%save all point pairs
    else
        pt_pairs=dlmread(name_points2);
        pt1=pt_pairs(:,3:4);
        pt2=pt_pairs(:,1:2);
    end

    my_displaymatch(imgbase,img2,pt1,pt2);
    figure(1);
    imshow(imgbase);hold on;
    plot(pt1(:,1),pt1(:,2),'*g');
    figure(2);
    imshow(img2);hold on;
    plot(pt2(:,1),pt2(:,2),'*g');

	%-------------------------------------------------------------------------%
	if Option_ransac ==true
        temp=randperm(num_point_pairs);
        idx=temp(1:floor(num_point_pairs*0.8));
		X1 = pt1(idx,:)'; X1(3,:) = 1 ;%base train
		X2 = pt2(idx,:)'; X2(3,:) = 1 ;%input
        X1_test = pt1(temp(floor(num_point_pairs*0.8)+1:num_point_pairs),:)'; X1_test(3,:) =1;%test
        X2_test = pt2(temp(floor(num_point_pairs*0.8)+1:num_point_pairs),:)'; X2_test(3,:) =1;
		[H,score, error, err_train] = homo_ransac(X1, X2, floor(num_point_pairs*0.8), false);%without outliers
        err_mean_train = sum(err_train)/length(err_train);
		var_train = (err_train-err_mean_train)*(err_train-err_mean_train)'/length(err_train);
		fprintf(' train: mean: %f\n', err_mean_train);
		fprintf(' train: variance: %f\n', var_train);
        %---------------calculate test error---------------
        X2_ = H * X1_test;
        du = X2_(1,:)./X2_(3,:) - X2_test(1,:)./X2_test(3,:) ;
        dv = X2_(2,:)./X2_(3,:) - X2_test(2,:)./X2_test(3,:) ;
        err_test = sqrt(du.*du + dv.*dv);
        %---------------test error done-------------
        err_mean_test = sum(err_test)/length(err_test);
		var_test = (err_test-err_mean_test)*(err_test-err_mean_test)'/length(err_test);
        fprintf(' test: mean error of mosaicing: %f\n', err_mean_test);
		fprintf(' test: variance error: %f\n', var_test);
		Homo = H;%from base to img2
        err_mean_record(ID) = err_mean_test;
        var_record(ID) = var_test;
        err_mean_train_record(ID) = err_mean_train;
        var_train_record(ID) = var_train;
    end
    if Option_tform ==true%perform cross validation
        %leave one out
        num_folds = num_point_pairs/2;
        for m=1:num_folds
            idx = true(num_point_pairs,1);
            idx(2*(m-1)+1)=false;
            idx(2*m)=false;
            pt1_train = pt1(idx,:);
            pt2_train = pt2(idx,:);
            X1 = pt1(idx,:)'; X1(3,:) = 1 ;%base train
            X2 = pt2(idx,:)'; X2(3,:) = 1 ;%input
            X1_test = pt1(~idx,:)'; X1_test(3,:) =1;%test
            X2_test = pt2(~idx,:)'; X2_test(3,:) =1;
            tform = cp2tform(pt2_train,pt1_train,'projective');%'similarity');% (input, base)input_points:M*2 containing x and y.
            param=tform.tdata.T;%p_base = p_input * param, it is from img2 to base image.
            param = param';%because most use this form : p_base = param * p_input
            H=inv(param);
            %---------------calculate train error---------------
            X2_ = H * X1;
            du = X2_(1,:)./X2_(3,:) - X2(1,:)./X2(3,:) ;
            dv = X2_(2,:)./X2_(3,:) - X2(2,:)./X2(3,:) ;
            err_train = sqrt(du.*du + dv.*dv);
            %---------------train error done-------------
            err_mean_train = sum(err_train)/length(err_train);
            var_train = (err_train-err_mean_train)*(err_train-err_mean_train)'/length(err_train);
            %---------------calculate test error---------------
            X2_ = H * X1_test;
            du = X2_(1,:)./X2_(3,:) - X2_test(1,:)./X2_test(3,:) ;
            dv = X2_(2,:)./X2_(3,:) - X2_test(2,:)./X2_test(3,:) ;
            err_test = sqrt(du.*du + dv.*dv);
            %---------------test error done-------------
            err_mean_test = sum(err_test)/length(err_test);
            var_test = (err_test-err_mean_test)*(err_test-err_mean_test)'/length(err_test);
            Homo = H;%from base to img2
            err_mean_train_all(m,1)=err_mean_train;
            err_mean_test_all(m,1)=err_mean_test;
            var_train_all(m,1)=var_train;
            var_test_all(m,1)=var_test;
        end
        fprintf(' cross validation train: mean: %f\n', mean(err_mean_train_all));
        fprintf(' cross validation train: variance: %f\n', mean(var_train_all));
        fprintf(' cross validation test: mean: %f\n', mean(err_mean_test_all));
        fprintf(' cross validation test: variance: %f\n', mean(var_test_all));
        err_mean_test_cv_record(ID) = mean(err_mean_test_all);%leave-one-out cross validation error
        var_test_cv_record(ID) = mean(var_test_all);
        err_mean_train_cv_record(ID) = mean(err_mean_train_all);
        var_train_cv_record(ID) = mean(var_train_all);
    end
    if Option_tform ==true%use all point pairs
        X1 = pt1'; X1(3,:) = 1 ;%base train
        X2 = pt2'; X2(3,:) = 1 ;%input
        tform = cp2tform(pt2,pt1,'projective');%'similarity');% (input, base)input_points:M*2 containing x and y.
        param=tform.tdata.T;%p_base = p_input * param, it is from img2 to base image.
        param = param';%because most use this form : p_base = param * p_input
        H=inv(param);
        %---------------calculate train error---------------
        X2_ = H * X1;
        du = X2_(1,:)./X2_(3,:) - X2(1,:)./X2(3,:) ;
        dv = X2_(2,:)./X2_(3,:) - X2(2,:)./X2(3,:) ;
        err_train = sqrt(du.*du + dv.*dv);
        %---------------train error done-------------
        err_mean_train = sum(err_train)/length(err_train);
        var_train = (err_train-err_mean_train)*(err_train-err_mean_train)'/length(err_train);
        fprintf(' -------------------train: mean: %f--------------------\n', err_mean_train);
        fprintf(' -------------------train: variance: %f----------------\n', var_train);
        Homo = H;%from base to img2
        err_mean_record(ID) = err_mean_train;
        var_record(ID) = var_train;
    end
    if err_mean_train+var_train <= thresh_max_error && err_mean_test_cv_record(ID)+var_test_cv_record(ID) <= 3*thresh_max_error
        %accept this frame and record
        ID_valid(ID) = true;
        Homos(3*(ID-1)+1:3*ID, :) = Homo;
    else
        fprintf('Error. Check the warning from cross validation. Point file name: %s\n', name_points2(end-16:end));
        if flag_second_scan == true%repeat this image
            fprintf('Repeat Image %s:\n', name_img2);
            ID=ID-1;
        end
    end
    ID=ID+1;
end
fprintf('average mean for this dataset: %f\n', mean(err_mean_record(ID_valid)));
fprintf('average variance for this dataset: %f\n', mean(var_record(ID_valid)));
fprintf('average cross validation test mean for this dataset: %f\n', mean(err_mean_test_cv_record(ID_valid)));
fprintf('average cross validation test variance for this dataset: %f\n', mean(var_test_cv_record(ID_valid)));
disp('output homography (img2 to img_base) and statistics for compare_performance_all_realimages.m\n');
name_Homos=[path_dataset '\Homos.txt'];
name_statistics=[path_dataset '\Statistics.txt'];
statistics(:,1)=double(ID_valid);
statistics(:,2)=err_mean_record;
statistics(:,3)=var_record;
statistics(:,4)=err_mean_train_cv_record;
statistics(:,5)=var_train_cv_record;
statistics(:,6)=err_mean_test_cv_record;
statistics(:,7)=var_test_cv_record;

dlmwrite(name_Homos, Homos);
dlmwrite(name_statistics, statistics);

end












