function [segments, segments_half, segments_fp_index, segments_half_fp_index]=RBSD(Iridge, feature_points, ridges, hessian_angle)
%ridgeness based vessel tracing
%input:
%    Ivessel: frangi vesselness image
%    feature_points: n*3, 1st for x, 2nd for y, 3rd for score,
%        which is the number of connected component
%    ridges: cell n*1, each cell contains the ridge points associated
%        with the branch point, same format as connected_peaks
%    hessian_angle: eigenvector (smallest eigenvalue, along vessel), in angle format atan2, use cos and sin to recover. 
%output:
%    segments: {k}n*2  k number of segments, each cell contains a segment, each row [x,y] is a point. first
%        rwo is head feature point, last row is tail feature point
%    segments_fp_index: n*2 the index of feature points in the corresponding
%        segment in feature_points
%    segments_half: m*2 segments_half_fp_index  same thing as above but
%    for segment which only have one end as feature point
%todo: consider of using hessian angle in future

%initialization of parameters----------------------------------------------
radius_fp=8;% the width of square that belongs to a feature point
max_iteration = 1000;
thresh_ridge = 0;
mask_ridge = Iridge>thresh_ridge;
imsz = size(Iridge);
%construct feature distribution matrix for the usage of checking whether
%another feature point is nearby
mat_feature = zeros(imsz);
mask_fp_area = zeros(imsz(1), imsz(2));%mask the local area of a feature point
for i=1:size(feature_points,1)
    x=feature_points(i,1);
    y=feature_points(i,2);
    x=floor(x+0.5);
    y=floor(y+0.5);%make sure feature point does go to the very boundary
    mat_feature(y,x) = i;%record the index of feature point
    for j=-radius_fp:radius_fp
        mask_fp_area(y+j,x-radius_fp:x+radius_fp)=i;%number is index of fp
    end
end
feature_visited = zeros(size(feature_points,1),4);
%n*4 mark whether each feature point has been visited 4 for potentially 4 directions. 0 not visited, 1 visited
%this is to avoid twice visited of one segment from two sides.
peak_visited = zeros(imsz(1), imsz(2));%mark the visited ridge point for chain code purpose
segments = {};      %all segments lie here
num_seg = 0;        %the number of segments
segments_fp_index = [];%n*2 each row, store the index of head and tail feature points
segments_half = {}; %segments without a tail
num_seg_half = 0;
segments_half_fp_index = [];
neighborsx = [-1 0 1 -1 1 -1 0 1];
neighborsy = [-1 -1 -1 0 0 1 1 1];
%main iteration------------------------------------------------------------
for i=1:size(feature_points,1)%scan each feature point
    x=feature_points(i,1);
    y=feature_points(i,2);
    ind_tail=-1;%initialization of the tail index
    for j=1:3 %each feature point has three directions
        if feature_visited(i,j) == 1
            continue;%skip if has been visited from the other end
        end
        feature_visited(i,j) = 1;%mark featurepoint_i j-th direction as visited
        x1=ridges{i}(1,j);
        y1=ridges{i}(2,j);
        flag_forwarding_peak = false;
        flag_fp_found = false;
        segment_try = [];%initialize a new segment
        segment_try(1,1) = x;%insert the head: feature point
        segment_try(1,2) = y;
        segment_try(2,1) = x1;%insert the first middle point
        segment_try(2,2) = y1;
%---check to see whether [xc yc] has neighbor which is forwarding peak-----
        k=1;
        dis_max=0;
        %first visit [xc yc] and its neighbors, find and check whether the
        %furthest one is forwarding
        peak_visited(y1,x1)=1;
        for m=1:8
            nx = x1+neighborsx(m);
            ny = y1+neighborsy(m);
            if mask_ridge(ny,nx)==1
                peak_visited(ny,nx)=1;
                dis = (nx-x)^2+(ny-y)^2;
                if dis>dis_max
                    dis_max=dis;
                    peak_max = m;
                end
                neig_peaks(k) = m;%remove later
                k=k+1;%remove later
            end
        end
        %move a step, so that next time only one forwarding direction is left.
        %set the found neighbor as current [xc yc]
        if dis_max >0
            xc = x1+neighborsx(peak_max);
            yc = y1+neighborsy(peak_max);%the first iteration
            %check whether [xc yc] is a forwarding peak
            for m=1:8
                nx = xc+neighborsx(m);
                ny = yc+neighborsy(m);
                if mask_ridge(ny,nx)==1 && peak_visited(ny,nx)==0
                    flag_forwarding_peak=true;%it has unvisited neighbors
                    break;
                end
            end
            segment_try(3,1) = xc;
            segment_try(3,2) = yc;
            seg_len = 3;
        else
            %disp('Warning! This is a feature point which does not have a forwarding neighbor. This segment will not be recorded.\n')
            continue;%in case somehow a bad feature point
        end
        iteration = 0;
%decision is made based on the fact that: for single pixel vessel, each
%step there will only be at most one neighbor that is forwarding peak
%forwarding pixel: a white pixel which has unvisited neighbors.
%num of forwarding peak: 0 end of a segment; 1 in the middle of a segment;
%>1 not single width, either a corner is missing or it is specular:
%randomly pick one forwarding peak.
%flag_forwarding_peak==true => assert [xc yc] should at least have one unvisited neighbor
        while(flag_forwarding_peak==true && flag_fp_found==false && iteration<max_iteration)%this loop is trying to find a new segment
%---Check to see whether [xc yc] has neighbor which is forwarding peak and decide the next movement: stop or continue-----
%flag_forwarding_peak shows [xc, yc] is a forwarding peak
            neig_peaks=0;
            forwarding_peaks=0;
            ind_tail=0;
            k=1;
            peak_visited(yc,xc)=1;
           %------First collect all unvisited neighbors and mark as visited
            for m=1:8
                nx = xc+neighborsx(m);
                ny = yc+neighborsy(m);
                if mask_ridge(ny,nx)==1 && peak_visited(ny,nx)==0
                    peak_visited(ny,nx)=1;
                    neig_peaks(k) = m;%find all unvisited neighbor peaks and mark as visited
                    k=k+1;
                end
            end
            num_peak_neig_unvisited = k-1;
            %assert(num_peak_neig_unvisited>=1);%assert [xc yc] should at least have one unvisited neighbor
            if num_peak_neig_unvisited==0
                disp('Error!! Should not come here. flag_forwarding_peak is not enhanced from last iteration.\n');
            end
            num_forwarding_destination=0;
            forwarding_destination=[0,0];%the destination of next step, the purpose is to decide whether to check tail or not
            num_forwarding_neigs=0;
            %---From all unvisited neighbor peaks, find the forwarding ones
            for n=1:num_peak_neig_unvisited  
                xc_n = xc+neighborsx(neig_peaks(n));
                yc_n = yc+neighborsy(neig_peaks(n));
                for m=1:8
                    nx = xc_n+neighborsx(m);
                    ny = yc_n+neighborsy(m);
                    if mask_ridge(ny,nx)==1 && peak_visited(ny,nx)==0 && (abs(nx-xc)>1 || abs(ny-yc)>1)%not the neighbor of [xc yc]
                        num_forwarding_neigs=num_forwarding_neigs+1;
                        forwarding_peaks(num_forwarding_neigs)=n;
                        if forwarding_destination(1,1)~=nx || forwarding_destination(1,2)~=ny%if first or a new destination
                            num_forwarding_destination=num_forwarding_destination+1;
                            forwarding_destination(num_forwarding_destination,1)=nx;
                            forwarding_destination(num_forwarding_destination,2)=ny;
                        end
                    end
                end  
            end
            %-----make decision based on the number of forwarding neighbors
            if num_forwarding_destination ==0%no forwarding peak, this peak is the end
                xc = xc+neighborsx(neig_peaks(1));
                yc = yc+neighborsy(neig_peaks(1));
                flag_forwarding_peak = false;%tell next iteration, [xc yc] is not forwarding
                seg_len = seg_len+1;
                segment_try(seg_len,1) = xc;
                segment_try(seg_len,2) = yc;
            end
            if num_forwarding_destination >= 1%got the forwarding peak
                xc = xc+neighborsx(neig_peaks(forwarding_peaks(1)));
                yc = yc+neighborsy(neig_peaks(forwarding_peaks(1)));
                flag_forwarding_peak = true;
                seg_len = seg_len+1;
                segment_try(seg_len,1) = xc;
                segment_try(seg_len,2) = yc;
            end
%--------------check whether this is the end of the segment: another feature point is nearby
            flag_fp_found=false;
            if mask_fp_area(yc,xc)>0 && mask_fp_area(yc,xc)~=i%not the current fp
                ind_tail = mask_fp_area(yc,xc);
                flag_fp_found = true;
                seg_len = seg_len+1;
                segment_try(seg_len,1) = feature_points(ind_tail,1);
                segment_try(seg_len,2) = feature_points(ind_tail,2);%this is the last point in this segment
            end
            iteration = iteration+1;
        end
%------after each iteration of peak, construct segments and segments_half
        if flag_fp_found == true
            num_seg = num_seg+1;
            segments{num_seg} = segment_try;
            segments_fp_index(num_seg,1) = i;
            segments_fp_index(num_seg,2) = ind_tail;%ind_tail should be positive
            assert(ind_tail>0); %following is to mark this feature-direction as visited, so they won't be visited twice
            dis1 = norm(segment_try(end-1,1:2)' - ridges{ind_tail}(1:2,1));%now ind_tail should have value
            dis2 = norm(segment_try(end-1,1:2)' - ridges{ind_tail}(1:2,2));
            dis3 = norm(segment_try(end-1,1:2)' - ridges{ind_tail}(1:2,3));
            if dis1<=dis2 && dis1<=dis3
                feature_visited(ind_tail,1)=1;
            end
            if dis2<dis1 && dis2<=dis3
                feature_visited(ind_tail,2)=1;
            end
            if dis3<dis1 && dis3<dis2
                feature_visited(ind_tail,3)=1;
            end
        end
        if flag_fp_found == false && flag_forwarding_peak == false
            num_seg_half = num_seg_half+1;
            segments_half{num_seg_half} = segment_try;
            segments_half_fp_index(num_seg_half,1) = i;
        end
        if iteration>=max_iteration
            disp('Warning! Iteration exceeds maximum number.');
        end
    end
end  







