function [Iridge] = calc_ridgeness(img, Ivessel, sigma, hessian_angle)
%detect ridge and branch points based on the description from paper 
%TMI 2004 ridge-based vessel segmentation in color images of the retina
%img: original image
%sigma: The sigma of the gaussian kernel
%hessian_angle: hessian matrix vector corresponds to the largest eigenvalue [-pi, pi]
%finally, all bugs cleared. very clear ridge image

%since the input angle is along the vessel (small eigenvalue), plus pi/2
hessian_Ix = cos(hessian_angle+pi/2);
hessian_Iy = sin(hessian_angle+pi/2);%from angle to vector

gauss_filter = fspecial('gaussian', [5 5], sigma);%pay attention here, smoothing + ridge detection
img = imfilter(img, gauss_filter, 'replicate');

[Y,X]   = ndgrid(-round(3*sigma):round(3*sigma));%should be [Y,X] rather than [X,Y]
DGaussx = 1/(2*pi*sigma^2)*(-X/(sigma^2)) .* exp(-(X.^2 + Y.^2)/(2*sigma^2));
DGaussy = DGaussx';

Dx = imfilter(img,DGaussx,'conv');
Dy = imfilter(img,DGaussy,'conv');

margin = 3;
imsz = size(img);
mask_ridge = zeros(imsz);
%attention, I have spent a long time 
%dealing with image coordinate and normal xy coordinate
%[hessian_Ix, hessian_Iy] represents eigenvector, they are represented in normal xy coordinate

[X,Y]=meshgrid(1:imsz(2), 1:imsz(1));%[X,Y] = ndgrid(imsz);%
Gradx = interp2(Dx, X+hessian_Ix, Y+hessian_Iy, 'bilinear');
Grady = interp2(Dy, X+hessian_Ix, Y+hessian_Iy, 'bilinear');
innerprod1 = Gradx.*hessian_Ix + Grady.*hessian_Iy;
Gradx = interp2(Dx, X-hessian_Ix, Y-hessian_Iy, 'bilinear');
Grady = interp2(Dy, X-hessian_Ix, Y-hessian_Iy, 'bilinear');
innerprod2 = Gradx.*hessian_Ix + Grady.*hessian_Iy;
mask_ridge = innerprod1.*innerprod2 <0;%across the largest curveture direction, the gradient changes sign
Iridge = mask_ridge.*Ivessel;

flag_single_pixel_vesselness = false;
if flag_single_pixel_vesselness == true
    IRshift1 = interp2(Ivessel, X+hessian_Ix, Y+hessian_Iy, 'bilinear');
    IRshift2 = interp2(Ivessel, X-hessian_Ix, Y-hessian_Iy, 'bilinear');
    mask_ridge2 = Ivessel>IRshift1 & Ivessel>IRshift2;
    Iridge2 = mask_ridge2.*Ivessel;
    figure(49);
    imshow(mask_ridge2);
    figure(50);
    imshow(Iridge2, [0, 0.05]);
end
flag_single_pixel=true;
if flag_single_pixel == true%the major influcence on single and double is : size of smallest cluster
    IRshift1 = interp2(Iridge, X+hessian_Ix, Y+hessian_Iy, 'bilinear');
    IRshift2 = interp2(Iridge, X-hessian_Ix, Y-hessian_Iy, 'bilinear');
    mask_ridge = Iridge>IRshift1 & Iridge>IRshift2;
    Iridge = mask_ridge.*Iridge;
end










