function [feature_points, ridges] = VBCT(Ivessel, radius, DoH)
%Given Frangi Vesselness information, detect branching points
% input: 
%        Ivessel: vesselness image
%        hessian_angle: angle of eigenvector (smallest eigenvalue, along vessel), obtained from atan2, use cos and sin to recover. 
%        radius: n*1 radius used of circles used in VBCT
%        DoH: determinant of Hessian: lambda1*lambda2
% output: 
%        feature_points: n*3, 1st for x, 2nd for y, 3rd for score, which is the number of points in the connected component
%        ridges: cell n*1, each cell contains the ridge points associated
%           with the branch point, same format as connected_peaks
%Steps:
%1. scan all candidate points
%2. find connected components
%3. find the mean and score(number of point in the component)
%4. rank them and non-maximal suppression 

%initialization of parameters----------------------------------------------
distance_closest_feature_point = 10;
num_smallest_component = 3;
margin=radius*2;%margins that will not be processed
thresh_black = 0.01;
thresh_mu = 0.05;
k=1;
mark_candidate = DoH>thresh_mu & Ivessel>thresh_black ;
if max(max(Ivessel)>1)
  Ivessel = im2double(Ivessel);%convert to [0 1]
end
imsz = size(Ivessel);
imcenter = floor(imsz/2);
%get template circles------------------------------------------------------
[circley_temp, circlex_temp] = getmidpointcircle(imcenter(1), imcenter(2), radius(1));%pay attention to the x y coordinate here
circle_templatex1 = circlex_temp - imcenter(2);
circle_templatey1 = circley_temp - imcenter(1);%set center to (0,0)
[circley_temp, circlex_temp] = getmidpointcircle(imcenter(1), imcenter(2), radius(2));%pay attention to the x y coordinate here
circle_templatex2 = circlex_temp - imcenter(2);
circle_templatey2 = circley_temp - imcenter(1);%set center to (0,0)
%for each candidate, perform circle test-----------------------------------
for c=margin:imsz(2)-margin     %x
    for r=margin:imsz(1)-margin %y
        if mark_candidate(r,c)==0%skip if it is background point
            continue;
        end
        %first circle
        circlex1 = circle_templatex1 + c;
        circley1 = circle_templatey1 + r;%circle for this pixel[r c]
        num_pixel1 = length(circlex1);
        distribution1 = zeros(num_pixel1,1);
        for i = 1:num_pixel1
            px = circlex1(i);
            py = circley1(i);
            distribution1(i)=Ivessel(py, px);
        end
        [isbranch1, peaks1] = circle_test_vesselness(distribution1, Ivessel(r,c));
        %second circle
        circlex2 = circle_templatex2 + c;
        circley2 = circle_templatey2 + r;%circle for this pixel[r c]
        num_pixel2 = length(circlex2);
        distribution2 = zeros(num_pixel2,1);
        for i = 1:num_pixel2
            px = circlex2(i);
            py = circley2(i);
            distribution2(i)=Ivessel(py, px);
        end
        [isbranch2, peaks2] = circle_test_vesselness(distribution2, Ivessel(r,c));
        if isbranch1 || isbranch2 %&& flag_peak_checked
            branch_points(k,1)=c;
            branch_points(k,2)=r;
            len_peaks_allowed = 4;
            if isbranch1 %default: if both only store circle1
                len_peaks_allowed = min(len_peaks_allowed, length(peaks1));
                peak_points(2*(k-1)+1,1:len_peaks_allowed) = circlex1(peaks1(1:len_peaks_allowed));%format: each column: [x1,y1]'
                peak_points(2*(k-1)+2,1:len_peaks_allowed) = circley1(peaks1(1:len_peaks_allowed));
            else
                len_peaks_allowed = min(len_peaks_allowed, length(peaks2));
                peak_points(2*(k-1)+1,1:len_peaks_allowed) = circlex2(peaks2(1:len_peaks_allowed));%format: each column: [x1,y1]'
                peak_points(2*(k-1)+2,1:len_peaks_allowed) = circley2(peaks2(1:len_peaks_allowed));
            end
            if  len_peaks_allowed < 4  %if 3 peaks, zeros padding for unity
                peak_points(2*(k-1)+1,4) = 0;
                peak_points(2*(k-1)+2,4) = 0;
            end
            k=k+1;
            %plot(c,r,'*', 'Color', [0,255,255]/255);%[255,69,0]
        end
    end
end

[connected_components, tag_which_component, connected_peaks]=find_connected_components(branch_points, peak_points, imsz);

[feature_points, ridges]=nonmax_suppression(Ivessel, connected_components, connected_peaks, distance_closest_feature_point, num_smallest_component);















