function [segments, segments_half, ind_feature_points, ind_feature_points_half] = VBSD(Ivessel, feature_points, ridges, hessian_angle)
%for each branch point, trace its vessels
%input:
%    Ivessel: frangi vesselness image
%    feature_points: n*3, 1st for x, 2nd for y, 3rd for score,
%        which is the number of connected component
%    ridges: cell n*1, each cell contains the ridge points associated
%        with the branch point, same format as connected_peaks
%    hessian_angle: angle of eigenvector (smallest eigenvalue, along vessel), obtained from atan2, use cos and sin to recover. 
%output:
%    segments: {k}n*2  k number of segments, each cell contains a segment, each row [x,y] is a point. first
%        rwo is head feature point, last row is tail feature point
%    ind_feature_points: n*2 the index of feature points in the corresponding
%        segment in feature_points
%    segments_half: m*2 ind_feature_points_half  same thing as above but
%    for segment which only have one end as feature point

%initialization of parameters----------------------------------------------
hessian_Ix = cos(hessian_angle);
hessian_Iy = sin(hessian_angle);%from angle to vector
imsz = size(Ivessel);
%construct feature distribution matrix for the usage of checking whether
%another feature point is nearby
mat_feature = zeros(imsz);
for i=1:size(feature_points,1)
    x=feature_points(i,1);
    y=feature_points(i,2);
    x=floor(x+0.5);
    y=floor(y+0.5);%make sure feature point does go to the very boundary
    mat_feature(y,x) = i;%record the index of feature point
end
feature_visited = zeros(size(feature_points,1),4);
%n*4 mark whether each feature point has been visited 4 for potentially 4 directions. 0 not visited, 1 visited
%this is to avoid twice visited of one segment from two sides.
stepsize = 2;
vessel_size = 4;
segments = {};      %all segments lie here
num_seg = 0;        %the number of segments
ind_feature_points = [];%n*2 each row, store the index of head and tail feature points
segments_half = {}; %segments without a tail
num_seg_half = 0;
ind_feature_points_half = [];
radius_fp=8;
mask_fp_area = zeros(imsz(1), imsz(2));%mask the local area of a feature point
for i=1:size(feature_points,1)
    x=feature_points(i,1);
    y=feature_points(i,2);
    x=floor(x+0.5);
    y=floor(y+0.5);%make sure feature point does go to the very boundary
    mat_feature(y,x) = i;%record the index of feature point
    for j=-radius_fp:radius_fp
        mask_fp_area(y+j,x-radius_fp:x+radius_fp)=i;%number is index of fp
    end
end
%main iteration------------------------------------------------------------
for i=1:size(feature_points,1)%scan each feature point
    x=feature_points(i,1);
    y=feature_points(i,2);
    for j=1:3 %each feature point has three directions
        if feature_visited(i,j) == 1
            continue;%skip if has been visited from the other end
        end
        feature_visited(i,j) = 1;%mark featurepoint_i j-th direction as visited
        x1=ridges{i}(1,j);
        y1=ridges{i}(2,j);
        xc=x1;%xc,yc stands for the current point
        yc=y1;
        flag_is_peak = true;
        flag_is_tail = false;
        segment_try = [];%initialize a new segment
        segment_try(1,1) = x;%insert the head: feature point
        segment_try(1,2) = y;
        segment_try(2,1) = xc;%insert the first middle point
        segment_try(2,2) = yc;
        seg_len = 2;
        iteration = 0;
        while(flag_is_peak==true && flag_is_tail==false && iteration<1000)%this loop is trying to find a new segment
            dirtax = hessian_Ix( floor(yc+0.5), floor(xc+0.5));%%bilinear interpolation can't be used because a line has two directions
            dirtay = hessian_Iy( floor(yc+0.5), floor(xc+0.5));
            vec_outward = [xc-segment_try(seg_len-1,1), yc-segment_try(seg_len-1,2)];%the vector from last peak point to the current peak point
            if [dirtax dirtay]*vec_outward' < 0
                dirtax = -dirtax;
                dirtay = -dirtay;
            end
            assert([dirtax dirtay]*vec_outward' > 0)%guarantee the tracing is moving forward 
            center_line = [xc,yc]+stepsize*[dirtax, dirtay];
            steps_line = (-1*vessel_size:vessel_size)';%find the pixels that are perpendicular to vessel direction
            perp_line = repmat(center_line,length(steps_line),1) + repmat(steps_line,1,2).*repmat([-dirtay, dirtax], length(steps_line),1);
            distribution = interp2(Ivessel, perp_line(:,1), perp_line(:,2));%Ivessel(perp_line(:,2), perp_line(:,1));
            %check whether a peak can be found
            [is_found, ind_peak] = find_peak(distribution, interp2(Ivessel,xc,yc));
            if is_found
                x_peak = perp_line(ind_peak,1);
                y_peak = perp_line(ind_peak,2);
                flag_is_peak = true;
                xc = x_peak;
                yc = y_peak;
                seg_len = seg_len+1;
                segment_try(seg_len,1) = x_peak;
                segment_try(seg_len,2) = y_peak;
            else
                flag_is_peak = false;
            end
            %check whether this is the end of the segment: another feature
            %point is nearby
            flag_is_tail = false;
            ind_tail=0;
            if mask_fp_area(floor(yc+0.5), floor(xc+0.5))>0 && mask_fp_area(floor(yc+0.5), floor(xc+0.5))~=i%not the current fp
                ind_tail = mask_fp_area(floor(yc+0.5), floor(xc+0.5));
                flag_is_tail = true;
                seg_len = seg_len+1;
                segment_try(seg_len,1) = feature_points(ind_tail,1);
                segment_try(seg_len,2) = feature_points(ind_tail,2);%this is the last point in this segment
            end
            if flag_is_tail == true
                num_seg = num_seg+1;
                segments{num_seg} = segment_try;
                ind_feature_points(num_seg,1) = i;
                ind_feature_points(num_seg,2) = ind_tail;
                assert(ind_tail>0); %following is to mark this feature-direction as visited, so they won't be visited twice
                dis1 = norm(segment_try(end-1,1:2)' - ridges{ind_tail}(1:2,1));%now ind_tail should have value
                dis2 = norm(segment_try(end-1,1:2)' - ridges{ind_tail}(1:2,2));
                dis3 = norm(segment_try(end-1,1:2)' - ridges{ind_tail}(1:2,3));
                if dis1<=dis2 && dis1<=dis3
                    feature_visited(ind_tail,1)=1;
                end
                if dis2<dis1 && dis2<=dis3
                    feature_visited(ind_tail,2)=1;
                end
                if dis3<dis1 && dis3<dis2
                    feature_visited(ind_tail,3)=1;
                end
            end
            if flag_is_tail == false && flag_is_peak == false
                num_seg_half = num_seg_half+1;
                segments_half{num_seg_half} = segment_try;
                ind_feature_points_half(num_seg_half,1) = i;
            end
            iteration = iteration+1;
        end
    end
end







        
        
        
        
        

