function [connected_component_final, tag_which_component, connected_peaks]=find_connected_components(branch_points, peak_points, imsz)
%given the raw detected branch points, which are in the form of clusters
%find the connected component in 8-neighbors
%input: 
%       branch_points n*2  1st column for x, 2nd for y
%       peak_points:  2n*4 n the number of branch points, each branch point has
%           4 peak points (zero padding) from each circle. each column has format [x1, y1, x2, y2]'
%       imsz: image size
%output:
%       connected_component_final: cell {i}(j,k) ith component, each
%       component is a set i, which stores 2d points 1st for x 2nd for y
%       num_connected_component: number of components
%       size_component_final: size of points set in each component 
%       tag_which_component: represent component in 2d format
%       connected_peaks:     cell{i}, each cell is (2*k)*4 matrix, each
%           column element (for each k) [x1 y1 x2 y2]', each row has 4 num(0 padding if less)
connected_component={};
size_component = [];%indicates the size of the above cell which contains sets
num_connected_component = 0;
tag_which_component=zeros(imsz);%0 not assigned to any 
%component yet; positive: the index in connected_component
tag_is_branch = zeros(imsz);%0 not branch point, 1 yes
for i=1:size(branch_points,1)
    x = branch_points(i,1);
    y = branch_points(i,2);
    tag_is_branch(y,x) = 1;
end
for i=1:size(branch_points,1)
    x = branch_points(i,1);
    y = branch_points(i,2);
    index_component = -1;
    if tag_which_component(y,x)==0%if (x,y) not assigned to any component
        %check its 8 neighbors
        flag_found_one_neighbor = false;
        for m=-1:1
            for n=-1:1
                if tag_is_branch(y+m,x+n)==1 && tag_which_component(y+m, x+n)>0%if one neighbor is assigned to a component
                    index_component = tag_which_component(y+m,x+n);
                    flag_found_one_neighbor = true;
                    break;
                end
            end
            if flag_found_one_neighbor == true
                break;
            end
        end
        if flag_found_one_neighbor == false%if no neighbor is assigned yet, creat a new component
            num_connected_component = num_connected_component+1;
            index_component = num_connected_component;
            size_component(index_component) = 0;
        end
    else
        index_component = tag_which_component(y,x);
    end
    %till now, middle point or a neighbor should have been assigned to a
    %connected component (index_component). add all 9 points if not
    %assigned yet to component index_component
    for m=-1:1
        for n=-1:1
            if tag_is_branch(y+m,x+n)==1 && tag_which_component(y+m, x+n)<=0
                tag_which_component(y+m,x+n)=index_component;
                connected_component{index_component}(size_component(index_component)+1,1)=x+n;
                connected_component{index_component}(size_component(index_component)+1,2)=y+m;
                size_component(index_component) = size_component(index_component) +1;
            end
        end
    end
end
%because the same component might be seperated into diff indices, scan and
%merge them
pair_merge_component = [];
num_pair = 0;
for i=1:size(branch_points,1)
    x = branch_points(i,1);
    y = branch_points(i,2);
    for m=-1:1
        for n=-1:1
            if tag_which_component(y+m, x+n)~=tag_which_component(y,x) && tag_which_component(y,x)>0 && tag_which_component(y+m,x+n)>0             
                num_pair = num_pair+1;
                pair_merge_component(num_pair,1)=min(tag_which_component(y,x), tag_which_component(y+m,x+n));
                pair_merge_component(num_pair,2)=max(tag_which_component(y,x), tag_which_component(y+m,x+n));
            end
        end
    end
end
pair_merge_component_sorted = sortrows(pair_merge_component);%sort each row as a unit based on the first column 
equivalent_component = zeros(num_connected_component,1);%this serves as a linked list, positive numbers store the index
for i=1:num_pair%for equivalent component, link to the smallest component
    a=pair_merge_component(i,1);
    b=pair_merge_component(i,2);
    if equivalent_component(a)==0 %if b is the smallest 
        equivalent_component(b)=a;
    else %if not assigned to the smallest component
        equivalent_component(b)=equivalent_component(a);
    end
end
table_component_index_old2new = zeros(num_connected_component,1);%convert old component index to new (re-index for condensing)
previous = 0;
for i=1:num_connected_component
    if equivalent_component(i)==0
        table_component_index_old2new(i) = previous+1;
        previous=previous+1;
    else
        assert(i>equivalent_component(i));%because sorted
        table_component_index_old2new(i) = table_component_index_old2new(equivalent_component(i));%previous one should have index already
    end      
end

%update tag_which_component, 
num_connected_component=num_connected_component-sum(equivalent_component>0);
connected_component_final={};  %connected_component after merging
size_component_final = zeros(num_connected_component,1);%size of each set element in connected_component after merging
for i=1:size(branch_points,1)
    x = branch_points(i,1);
    y = branch_points(i,2);
    if tag_which_component(y,x)>0
        if equivalent_component(tag_which_component(y,x))>0
            tag_which_component(y,x) = equivalent_component(tag_which_component(y,x));
        end
        new_index = table_component_index_old2new(tag_which_component(y,x));%use the new component index
        tag_which_component(y,x) = new_index;
        size_component_final(new_index)=size_component_final(new_index)+1;
        connected_component_final{new_index}(size_component_final(new_index),1)=x;
        connected_component_final{new_index}(size_component_final(new_index),2)=y;
        k=size_component_final(new_index);
        connected_peaks{new_index}(2*(k-1)+1,:) = peak_points(2*(i-1)+1,:);
        connected_peaks{new_index}(2*(k-1)+2,:) = peak_points(2*(i-1)+2,:);
    end
end



