function [feature_points, ridges]=nonmax_suppression(Ivessel, connected_component, connected_peaks, distance_closest_feature_point, num_smallest_component)
%detect feature points and calculate the score based on the input connected
%component. Each feature point is the center of each component. This is
%because the branch points are in the form of connected component
%input: 
%       connected_component: cell {i}(j,k) ith component, each
%           component is a set, which stores 2d points 1st for x 2nd for y
%       connected_peaks: (4k)by4, one branch point contributes 4*4 matrix, 
%           where each column is [x1,y1, x2, y2]', which is a peak in each 
%           circle. To unify, each branch point is having 4 peaks, if 3, 
%           zeros are padded  
%       distance_closest_feature_point: threshold for non-maximal
%           suppression, the closest feature points distance
%output:
%       feature_points: n*3, 1st for x, 2nd for y, 3rd for score,
%           which is the number of connected component
%       ridges: cell n*1, each cell contains the ridge points associated
%           with the branch point, same format as connected_peaks
%todo: use the circle properties (difference of pixel) for better score
num_connected_component = length(connected_component);
for i=1:num_connected_component
    tmpx=0;
    tmpy=0;
    weight = 0;
    for j=1:size(connected_component{i},1)
        x = connected_component{i}(j,1);
        y = connected_component{i}(j,2);
        tmpx = tmpx + x*Ivessel(y,x);
        tmpy = tmpy + y*Ivessel(y,x);
        weight = weight + Ivessel(y,x);
    end
    centers(i,1) = tmpx/weight;
    centers(i,2) = tmpy/weight;
%     centers(i,1) = mean(connected_component{i}(:,1));%x
%     centers(i,2) = mean(connected_component{i}(:,2));%y
    centers(i,3) = size(connected_component{i},1);   %score 
end
%centers = sortrows(centers,-3);% rank the feature points based on the component size
%sort feature points and ridges points based on the scores
for i=1:size(centers,1)
    for j=i+1:size(centers,1)
        if centers(i,3)<centers(j,3)
            temp = centers(i,:);
            centers(i,:) = centers(j,:);
            centers(j,:) = temp;
            temp = connected_peaks{i};
            connected_peaks{i} = connected_peaks{j};
            connected_peaks{j} = temp;
        end
    end
end

k=1;
for i=1:num_connected_component
    flag_too_close = false;
    for j=1:i-1
        if norm(centers(i,1:2)-centers(j,1:2)) < distance_closest_feature_point
            flag_too_close = true;
            continue;
        end
    end
    if flag_too_close == false
        feature_points(k,:) = centers(i,:);
        ridges{k,1} = connected_peaks{i};
        k=k+1;
    end
end


index_goodfeature = feature_points(:,3)>=num_smallest_component;%each feature point has at least 5 connected component
feature_points = feature_points(index_goodfeature,:);
k=0;
for i=1:length(index_goodfeature)
    if index_goodfeature(i)==1
        k=k+1;
        ridges_final{k} = ridges{i};
    end
end
ridges = ridges_final;















    
