function [match,index_prj, index_det, index_large_error, myerror, count, count_valid]...
    =calculate_repeatability(feature_points2, points_prj2, thresh_distance, imsz1, Homo)
%calculate the repeatability of feature detector
%input: feautre_points2: feature points detected in second image, each row
%stores one point: [x y ...]
%       points_prj2: The reprojected points in the second image of the feature points detected in the first image
%       thresh_distance: decides whether a point repeats
%       imsz1: size of the first image
%output: 
%        match: n*2, n the number of matches 1st index in image1,2nd image2
%        index_prj: binary index, the ground truth repeated points 
%        index_det: binary index, the detected repeated points 
%        index_error: binary index, repeated points but with large error
%        myerror: stores the error of repeated points
%        count: the number of repeated points in the second image
%        count_valid: the number of valid detected points in the second
%        image
flag_show_boundary = false;%this one shows the triangle
num_fp2 = size(feature_points2,1);
index_prj = false(size(points_prj2,2),1);
index_det = false(num_fp2,1);
index_large_error = false(num_fp2,1);
gap = 5;
count = 0;
count_valid = 0;
myerror = [];
match=[];
thresh_large_error = 2.5;
Homo_inv = inv(Homo);
for i=1:num_fp2
    px=feature_points2(i,1);
    py=feature_points2(i,2);
    p2 = [px; py; 1];
    p1 = Homo_inv*p2;
    p1 = p1/p1(3);
    if p1(1)<=8*gap || p1(1)>=imsz1(2)-8*gap || p1(2)<=gap || p1(2)>=imsz1(1)-gap%if points are too close to the boundary
        if flag_show_boundary == true
            figure(2);
            hold on;
            plot(px, py, '^r');
        end
        continue;
    end
    count_valid = count_valid +1;
    dis_min = 100000;
    for j=1:size(points_prj2,2)
        qx=points_prj2(1,j);
        qy=points_prj2(2,j);
        dis = sqrt((px-qx)*(px-qx)+(py-qy)*(py-qy));
        if dis_min > dis
            dis_min = dis;
            j_min = j;
        end
    end
    if dis_min <= thresh_distance
        index_det(i) = true;    %mark a repeated point pair: (i, j_min)
        index_prj(j_min) = true;%mark this point in the second image
        count = count +1;
        myerror(1,count) = dis_min;
        match(count,1) = j_min;%first image
        match(count,2) = i;    %second image
        if dis_min>thresh_large_error
            index_large_error(i) = true;
        end
    end
end










