function data = compute_performance(id_method, path_data, format, thresh_specular, flag_overwrite)
%this is the main code for computation of each detector. It calls different 
%detector and caculate repeatability, accuracy, maching and so on.
%input: 
%    id_method: the index of method
%    path_data: the path of data
%    format: image format
%    thresh_specular: threshold of specular reflection
%    flag_overwrite: whether to overwrite the existing file
%output:
%    data: m*n m: number of images, n: each column records one measure,
%    including: repeatability, correctness and so on (see the end)        
%
%branching point: VBCT, RBCT, sofka
%with socres:     FAST, AFD, Hessian, HessianLaplace
%without scores:  SIFT, Harris, ShiTomashi
%vessel points (except branching points) in VBSD and RBSD do not have score
%-----------------first of all, choose one method-------------------------
flag_VBCT = false;
flag_RBCT = false;
flag_SIFT = false;
flag_FAST = false;
flag_sofka = false;
flag_AFD = false;
flag_Hessian = false;
flag_HessianLaplace = false;
flag_Harris = false;
flag_ShiTomashi = false;
flag_segments2points = false;
flag_VBSD = false;
flag_RBSD = false;

if(id_method == 1)   
    flag_VBCT = true;         
    fprintf('--------------------Method: VBCT--------------------------------------\n');
end
if(id_method == 2)   
    flag_RBCT = true;         
    fprintf('--------------------Method: RBCT--------------------------------------\n');
end
if(id_method == 3)   
    flag_SIFT = true;           
    fprintf('--------------------Method: SIFT--------------------------------------\n');
end
if(id_method == 4)   
    flag_FAST = true;           
    fprintf('--------------------Method: FAST--------------------------------------\n');
end
if(id_method == 5)   
    flag_sofka = true;          
    fprintf('--------------------Method: sofka--------------------------------------\n');
end
if(id_method == 6)   
    flag_AFD = true;    
    fprintf('--------------------Method: AFD---------------------------------\n');
end
if(id_method == 7)   
    flag_Hessian = true;        
    fprintf('--------------------Method: Hessian--------------------------------------\n');
end
if(id_method == 8)   
    flag_HessianLaplace = true; 
    fprintf('--------------------Method: HessianLaplace-------------------------------\n');
end
if(id_method == 9)   
    flag_Harris = true;         
    fprintf('--------------------Method: Harris--------------------------------------\n');
end
if(id_method == 10)  
    flag_ShiTomashi = true;     
    fprintf('--------------------Method: ShiTomashi-----------------------------------\n');
end
if(id_method == 16)  
    flag_VBSD = true;
    flag_segments2points = true;
    fprintf('--------------------Method: VBSD--------------------------------------\n');
end
if(id_method == 17)  
    flag_RBSD = true;
    flag_segments2points = true;
    fprintf('--------------------Method: RBSD--------------------------------------\n');
end
%---------------------set corresponding flag -----------------------------
flag_sparse_inherent = false;%integrate the nonmaximum into the code
thresh_distance = 3.5;
if flag_VBCT==true||flag_RBCT==true||flag_sofka==true||flag_FAST==true
    flag_do_sparse_outside=false;%branching points are sparse already
    if flag_FAST==true
        flag_sparse_inherent = true;%only FAST integrates the nonmaximum into the code
    end
else
    flag_do_sparse_outside = true;%use the provided scores and do nonmaximum suppression, if none randomly pick points
end


flag_visualization=false;
flag_SSD_matching = true;
flag_use_half_segments = false;
win_half = 5;
if ~exist('thresh_specular', 'var')
    thresh_specular = 255;
end
thresh_peak_hessian = 0.0008;%0.0005~0.001
thresh_edge_hessian = 10;
thresh_peak_sift = 0.008;%0.006 %0.005~0.007
thresh_edge_sift = 10;
thresh_fast = 14;   %17    %15~20
max_corner = 7000;
max_feat_points = 7000;%maximum number of feature points
thresh_sensitivityfactor = 0.1;%range(0 0.25)
thresh_qualitylevel = 0.001;%range(0, 1);
thresh_anistropic_str = '0.2'; %0.1~0.3
%-----------------load database--------------------------------------------
name_fmt = ['img*.' format];
struct_dir = dir(fullfile(path_data, name_fmt));
num_img = length(struct_dir);
img_name_all = cell(num_img,1);
for i=1:num_img
    img_name_all{i} = struct_dir(i).name;
end
Homos = dlmread([path_data '\Homos.txt']);%each three rows stores one homography mapping
Statistics = dlmread([path_data '\Statistics.txt']);%check calculate_homos.m
name_img1 = [path_data '\base.' format];
%-----------------initialization-------------------------------------------
num_img_valid = sum(Statistics(:,1));
repeatability = zeros(num_img_valid,1);
err_mean_out = zeros(num_img_valid,1);
variance_out = zeros(num_img_valid,1);
num_correct = zeros(num_img_valid,1);
num_total_match = zeros(num_img_valid,1);
correctness = zeros(num_img_valid,1);
num_fp_out1 = zeros(num_img_valid,1);
num_fp_out2 = zeros(num_img_valid,1);
%-----------------------calculate image1 first----------------------------
I=imread(name_img1);
if strcmp(path_data(end-4:end), 'clip3')
    I=I(:, 130:1100, :);%this is to cut non-planar part only used for pelvis dataset
end
I=im2double(I);
grayI = rgb2gray(I);
imsz1=size(I);
%------------------detect feature points in first image--------------------
if flag_VBCT == true
    radius = [11,7];
    [feature_points1] = detector_VBCT(I, radius, thresh_specular);  %n*3, 1st for x, 2nd for y, 3rd for score,
end
if flag_VBSD == true
    radius = [11,7];
    [feature_points1, segments1, segments_half1, segments_fp_index1, segments_half_fp_index1, Ivessel1] = detector_VBSD(I, radius, thresh_specular);  %n*3, 1st for x, 2nd for y, 3rd for score,
end
if flag_SIFT == true
    [fa, da] = vl_sift(single(rgb2gray(I)), 'EdgeThresh', thresh_edge_sift, 'PeakThresh', thresh_peak_sift) ;
    %[fa, da] = vl_covdet(single(rgb2gray(I)), 'method', 'DoG', 'verbose', 'PeakThreshold', 0.005, 'EdgeThreshold', 10);
    feature_points1 = fa';
    feature_points1(:,3)=(1:size(feature_points1,1))';%Attention! SIFT does not provide scores, which it should have
    fa = fa';
end
if flag_FAST == true
    if flag_sparse_inherent == true;
        [feature_points1, scores1]=fast9(grayI*255,thresh_fast,1,flag_sparse_inherent,win_half);
    else
        [feature_points1, scores1]=fast9(grayI*255,thresh_fast,1);
    end
end
if flag_RBCT == true
    radius = [7,5];
    [feature_points1]=detector_RBCT(I, radius, thresh_specular);
end
if flag_RBSD == true
    radius = [7,5];
    [feature_points1, segments1, segments_half1, segments_fp_index1, segments_half_fp_index1]=detector_RBSD(I, radius, thresh_specular);
end
if flag_sofka == true
    img_name_dos1 = ['"' name_img1 '"'];
    if strcmp(path_data(end-4:end), 'clip3')
        name_save = [path_data '\test1.bmp'];%clip3, img1 has been cut, so save it and then load it
        imwrite(I, name_save, 'bmp');
        img_name_dos1 =['"' name_save '"'];
    end
    path_save = path_data;
    path_save_dos1 = ['"' path_save '"'];
    str_cmd1 = ['vessel_tracer_sofka\vessel_tracer_ALL ' img_name_dos1 ' matched 1.0 -s 1 -d ' path_save_dos1];
    dos(str_cmd1);%calling the executable
    file_name1=[path_save '\trace_matched1.txt'];
    feature_points1 = read_sofka_file(file_name1);
end
if flag_AFD == true
    img_name_dos1 = ['"' name_img1 '"'];
    if strcmp(path_data(end-4:end), 'clip3')
        name_save = [path_data '\test1.bmp'];%clip3, img1 has been cut, so save it and then load it
        imwrite(I, name_save, 'bmp');
        img_name_dos1 =['"' name_save '"'];
    end
    str_cmd1 = ['Anisotropic\anisotropic -cpu -i ' img_name_dos1 ' 1.5 1.44 ' thresh_anistropic_str ' 7 1'];
    dos(str_cmd1);%calling the executable
    file_name1='cpuImgOut_0.txt';
    [feature_points1, scores1] = read_anisotropic_file(file_name1);
    feature_points1(:,3)=scores1;%overwrite the 3th column to be the score
%     if flag_sparse_inherent == true
%         [feature_points1, cornerness1] = nonmax_suppression_general(feature_points1, cornerness1, win_half, imsz1);
%     end
end
if flag_Hessian == true
    [frames1, descrs1, info1] = vl_covdet(single(rgb2gray(I)), 'method', 'Hessian', 'verbose', 'PeakThreshold', thresh_peak_hessian, 'EdgeThreshold', thresh_edge_hessian);
    feature_points1 = frames1';
    scores1=info1.peakScores';
    feature_points1(:,3)=scores1;%overwrite the 3th column to be the score
end
if flag_HessianLaplace == true
    [frames1, descrs1, info1] = vl_covdet(single(rgb2gray(I)), 'method', 'HessianLaplace', 'verbose', 'PeakThreshold', thresh_peak_hessian, 'EdgeThreshold', thresh_edge_hessian);
    feature_points1 = frames1';
    scores1=info1.peakScores';
    feature_points1(:,3)=scores1;%overwrite the 3th column to be the score
end
if flag_Harris == true
    feature_points1 = corner(grayI, 'Harris', max_corner, 'QualityLevel', thresh_qualitylevel, 'SensitivityFactor', thresh_sensitivityfactor);%MinimumEigenvalue, QualityLevel 0 < Q < 1, SensitivityFactor 0 < K < 0.25
    feature_points1(:,3)=(1:size(feature_points1,1))';%Attention! matlab implementation of Harris does not provide scores, which it should have
end
if flag_ShiTomashi == true
    feature_points1 = corner(grayI, 'MinimumEigenvalue', max_corner, 'QualityLevel', thresh_qualitylevel);%MinimumEigenvalue, QualityLevel 0 < Q < 1, SensitivityFactor 0 < K < 0.25
    feature_points1(:,3)=(1:size(feature_points1,1))';%Attention! matlab implementation of Harris does not provide scores, which it should have
end
if flag_segments2points == true
    num_seg1=size(segments1,2);
    num_seg_half1=size(segments_half1,2);
    num_fp1 = size(feature_points1,1);
    count_add = num_fp1;
    for m=1:num_seg1
        one_seg = segments1{m};
        for n=2:size(one_seg,1)-1
            count_add=count_add+1;
            feature_points1(count_add,1)=one_seg(n,1);
            feature_points1(count_add,2)=one_seg(n,2);
            feature_points1(count_add,3)=0;
        end
    end
    if flag_use_half_segments == true
        for m=1:num_seg_half1
            one_seg = segments_half1{m};
            for n=2:size(one_seg,1)-1
                count_add=count_add+1;
                feature_points1(count_add,1)=one_seg(n,1);
                feature_points1(count_add,2)=one_seg(n,2);
                feature_points1(count_add,3)=0;
            end
        end
    end
end

%----------remove points that are not in the planar area.------------------
row_min=1;
row_max=imsz1(1);
col_min=1;
col_max=imsz1(2);
flag_cut_nonplanar = false;
if strcmp(path_data(end-4:end), 'clip1')
    col_min=180;
    row_max=580;
    flag_cut_nonplanar = true;
end
if strcmp(path_data(end-4:end), 'clip2')
    row_max=630;
    flag_cut_nonplanar = true;
end
if strcmp(path_data(end-4:end), 'clip4')
    col_max = 850;
    flag_cut_nonplanar = true;
end
if strcmp(path_data(end-14:end), 'clip5')
    row_min=100;
    %col_max = 530;
    flag_cut_nonplanar = true;
end
if flag_cut_nonplanar == true
    temp=feature_points1;
    feature_points1=[];
    k=0;
    for m=1:size(temp,1)
        x=temp(m,1);
        y=temp(m,2);
        if col_min<=x && x<=col_max && row_min<=y && y<=row_max
            k=k+1;
            feature_points1(k,:)=temp(m,:);%only points within the planar area are copied
        end
    end
end
num_fp1 = size(feature_points1,1);
%----------remove non-planar points done---------------------------------

%-----------------------------main iteration-----------------------------
points_prj2_all = {};
i=0;%valid image index
for ID=1:num_img
    close all;
    name_img2 = [path_data '/' img_name_all{ID}];
    %load homography data, which has been generated from mosaicing code----
    ID_valid = Statistics(ID,1);
    Homo = Homos(3*(ID-1)+1:3*ID, :);%homo from img1 to img2
    if ID_valid ~= 1%only process the image with valid ID.
        continue;
    end
    i=i+1;
    %if the image has valid homography
    J=imread(name_img2) ;
    J=im2double(J);
    grayJ = rgb2gray(J);
    imsz2=size(J);
    figure(20);
    imshow(I);
%------------------detect feature points-----------------------------------
    if flag_VBCT == true
        radius = [11,7];
        [feature_points2] = detector_VBCT(J, radius, thresh_specular); %ridge: n*8, angles: n*4  -1 for invalid value
    end
    if flag_VBSD == true
        radius = [11,7];
        [feature_points2, segments2, segments_half2, segments_fp_index2, segments_half_fp_index2, Jvessel] = detector_VBSD(J, radius, thresh_specular); %ridge: n*8, angles: n*4  -1 for invalid value
    end
    if flag_SIFT == true
        [fb, db] = vl_sift(single(rgb2gray(J)), 'EdgeThresh', thresh_edge_sift, 'PeakThresh', thresh_peak_sift) ;
        feature_points2 = fb';
        feature_points2(:,3)=(1:size(feature_points2,1))';%attention SIFT does not provide scores
        fb=fb';
        [matches, scores] = vl_ubcmatch(da, db, 3.0) ;
        % Create a new image showing that two images side by side.
        im3 = appendimages(I,J);
        figure(3);
        colormap('gray');
        imagesc(im3);
        hold on;
        title('SIFT feature matching');
        cols1 = size(I,2);
        for j = 1: size(matches,2)
            line([fa(matches(1,j),1) fb(matches(2,j),1)+cols1], ...
                 [fa(matches(1,j),2) fb(matches(2,j),2)], 'Color', 'c');
        end
        hold off;
        num = size(matches,2);
        fprintf('SIFT found %d matches.\n', num);
    end
    if flag_FAST == true
        if flag_sparse_inherent == true;
            [feature_points2, scores2]=fast9(grayJ*255,thresh_fast,1,flag_sparse_inherent,win_half);
        else
            [feature_points2, scores2]=fast9(grayJ*255,thresh_fast,1);
        end
    end
    if flag_RBCT == true
        radius = [7,5];
        [feature_points2]=detector_RBCT(J, radius, thresh_specular);
    end
    if flag_RBSD == true
        radius = [7,5];
        [feature_points2, segments2, segments_half2, segments_fp_index2, segments_half_fp_index2]=detector_RBSD(J, radius, thresh_specular);
    end
    if flag_sofka == true%note that sometimes sofka executable crashes
        img_name_dos2 = ['"' name_img2 '"'];
        path_save = path_data;
        path_save_dos2 = ['"' path_save '"'];
        str_cmd2 = ['vessel_tracer_sofka\vessel_tracer_ALL ' img_name_dos2 ' matched 1.0 -s 2 -d ' path_save_dos2];
        dos(str_cmd2);%calling the executable
        file_name2='trace_matched2.txt';
        feature_points2 = read_sofka_file(file_name2);
        figure(7);
        imshow(J);
        hold on;
        plot(feature_points2(:,1), feature_points2(:,2),'.g');
    end
    if flag_AFD == true
        img_name_dos2 = ['"' name_img2 '"'];
        str_cmd2 = ['Anisotropic\anisotropic -cpu -i ' img_name_dos2 ' 1.5 1.44 ' thresh_anistropic_str ' 7 1'];
        dos(str_cmd2);%calling the executable
        file_name2='cpuImgOut_0.txt';%this is generated from the binary
        [feature_points2, scores2] = read_anisotropic_file(file_name2);
        feature_points2(:,3)=scores2;%overwrite the 3th column to be the score
%         if flag_sparse_inherent == true
%             [feature_points2, cornerness2] = nonmax_suppression_general(feature_points2, cornerness2, win_half, imsz2);
%         end
    end
    if flag_Hessian == true
        [frames2, descrs2, info2] = vl_covdet(single(rgb2gray(J)), 'method', 'Hessian', 'verbose', 'PeakThreshold', thresh_peak_hessian, 'EdgeThreshold', thresh_edge_hessian);
        scores2=info2.peakScores;
        feature_points2 = frames2';
        feature_points2(:,3)=scores2';%overwrite the 3th column to be the score
    end
    if flag_HessianLaplace == true
        [frames2, descrs2, info2] = vl_covdet(single(rgb2gray(J)), 'method', 'HessianLaplace', 'verbose', 'PeakThreshold', thresh_peak_hessian, 'EdgeThreshold', thresh_edge_hessian);
        scores2=info2.peakScores;
        feature_points2 = frames2';
        feature_points2(:,3)=scores2';%overwrite the 3th column to be the score
    end
    if flag_Harris == true
        feature_points2 = corner(grayJ, 'Harris', max_corner, 'QualityLevel', thresh_qualitylevel, 'SensitivityFactor', thresh_sensitivityfactor);%MinimumEigenvalue, QualityLevel 0 < Q < 1, SensitivityFactor 0 < K < 0.25
        feature_points2(:,3)=(1:size(feature_points2,1))';%attention matlab implementation of Harris does not provide scores, which it should have
    end
    if flag_ShiTomashi == true
        feature_points2 = corner(grayJ, 'MinimumEigenvalue', max_corner, 'QualityLevel', thresh_qualitylevel);%MinimumEigenvalue, QualityLevel 0 < Q < 1, SensitivityFactor 0 < K < 0.25
        feature_points2(:,3)=(1:size(feature_points2,1))';%attention matlab implementation of Harris does not provide scores, which it should have
    end
    if flag_segments2points == true
        num_seg2=size(segments2,2);
        num_seg_half2=size(segments_half2,2);
        num_fp2 = size(feature_points2,1);
        count_add = num_fp2;
        for m=1:num_seg2
            one_seg = segments2{m};
            for n=2:size(one_seg,1)-1
                count_add=count_add+1;
                feature_points2(count_add,1)=one_seg(n,1);
                feature_points2(count_add,2)=one_seg(n,2);
                feature_points2(count_add,3)=0;
            end
        end
        if flag_use_half_segments == true
            for m=1:num_seg_half2
                one_seg = segments_half2{m};
                for n=2:size(one_seg,1)-1
                    count_add=count_add+1;
                    feature_points2(count_add,1)=one_seg(n,1);
                    feature_points2(count_add,2)=one_seg(n,2);
                    feature_points2(count_add,3)=0;
                end
            end
        end
    end
%-------------------features should have been done now---------------------
    %first of all, make sure number of feature points is less than maximum
    num_fp1 = size(feature_points1,1);
    num_fp2 = size(feature_points2,1);
    num_fp1_ori = num_fp1;
    num_fp2_ori = num_fp2;
    fprintf('Number of points detected in I(no sparsing): %d\n', num_fp1);
    fprintf('Number of points detected in J(no sparsing): %d\n', num_fp2);
    if num_fp1 > max_feat_points% fast, anisotropic, corners are ordered by rows, sift Hessian by columns
        ind_sel = randperm(num_fp1);
        if flag_FAST == true || flag_SIFT ==true || flag_Hessian==true || flag_HessianLaplace==true  || flag_AFD==true
            ind_sel = ind_sel(1:max_feat_points);
        else
            ind_sel = 1:max_feat_points;
        end
        feature_points1 = feature_points1(ind_sel,:);
    end
    if num_fp2 > max_feat_points%
        ind_sel = randperm(num_fp2);
        if flag_FAST == true || flag_SIFT ==true|| flag_Hessian==true || flag_HessianLaplace==true  || flag_AFD==true
            ind_sel = ind_sel(1:max_feat_points);
        else
            ind_sel = 1:max_feat_points;
        end
        feature_points2 = feature_points2(ind_sel,:);
    end
    %if sparse feature is required, use the provided scores
    if flag_do_sparse_outside == true && flag_sparse_inherent == false%if inherent nonmaximum suppression is not executed
        if ID==1%it is the same, only one time is enough
            if flag_RBSD || flag_VBSD || flag_AFD || flag_Hessian || flag_HessianLaplace || flag_SIFT || flag_Harris || flag_ShiTomashi
                scores1=feature_points1(:,3);
            end
            [feature_points1,scores1]= nonmax_suppression_general(feature_points1, scores1, win_half, imsz1);
        end
        if flag_RBSD || flag_VBSD || flag_AFD || flag_Hessian || flag_HessianLaplace || flag_SIFT || flag_Harris || flag_ShiTomashi
            scores2=feature_points2(:,3);
        end
        [feature_points2,scores2]= nonmax_suppression_general(feature_points2, scores2, win_half, imsz2);
    end
    num_fp1 = size(feature_points1,1);
    num_fp2 = size(feature_points2,1);
    points_prj2=Homo*[feature_points1(:,1)'; feature_points1(:,2)'; zeros(1,num_fp1)+1];
    points_prj2=points_prj2./repmat(points_prj2(3,:),3,1);

%--------------------calculate repeatability-------------------------------    
    points_prj2_all{i} = points_prj2;%save the data for calculation of repeatability
    feature_points2_all{i} = feature_points2;
    Homo_all{i} = Homo;
    thresh_distance_all{i} = thresh_distance;

    [match_rept, index_prj, index_det, index_large_error, myerror, count, count_valid]...
        =calculate_repeatability(feature_points2, points_prj2, thresh_distance, imsz1, Homo);
    %sum((myerror<0.5))
    repeatability(i) = count/min(count_valid, num_fp1);
    fprintf('Number of points detected in I(after sparsing): %d\n', num_fp1);
    fprintf('Number of points detected in J(after sparsing): %d\n', num_fp2);
    fprintf('The repeatability is %f\n', repeatability(i));
    fprintf('The average error is %f\n', sum(myerror)/count);
    if count~=0
        err_mean_out(i) = sum(myerror)/count;
        variance_out(i) = (myerror - sum(myerror)/count)*(myerror - sum(myerror)/count)'/count;
    else
        err_mean_out(i) = NaN;
        variance_out(i) = NaN;
    end
    num_fp_out1(i) = num_fp1_ori;
    num_fp_out2(i) = num_fp2_ori;
    fprintf('The variance is %f\n', variance_out(i));

    figure(6);
    imshow(I);
    hold on;
    title('Detected feature points in image I');
    plot(feature_points1(:,1), feature_points1(:,2),'.g');
    figure(7);
    imshow(J);
    hold on;
    title('Detected feature points in image J');
    plot(feature_points2(:,1), feature_points2(:,2),'.g');
    figure(2);
    imshow(J);
    hold on;
    title('Repeated feature points in image J');
    plot(feature_points2(index_det,1), feature_points2(index_det,2), 'go', 'MarkerSize',5, 'LineWidth',3);
    plot(points_prj2(1,index_prj), points_prj2(2,index_prj),'*r');
    plot(feature_points2(index_large_error,1), feature_points2(index_large_error,2), 'o', 'MarkerSize',5, 'LineWidth',3, 'Color', 'c');
%-----------------------------visualization tool for mapping--------------%
    if flag_visualization == true && (flag_VBCT == true || flag_RBCT == true)
        visualization_test_accuracy(Ivessel, Jvessel, feature_points1, feature_points2, points_prj2, ...
            index_det, index_prj, index_large_error, Homo)
    end

%---------------------------ground truth affine warp SSD matching---------%
    if flag_SSD_matching == true
        [match_ssd, match_correct,count_match, count_correct, min_value]=calculate_matching_correctness(grayI, ...
        grayJ, feature_points1, feature_points2, points_prj2, Homo, thresh_distance);
        num_correct(i) = count_correct;
        num_total_match(i) = count_match;
        correctness(i) = count_correct/count_match;
        im3 = appendimages(I,J);
        figure(8);
        colormap('gray');
        imagesc(im3);
        hold on;
        cols1 = size(I,2);
        for j = 1: length(match_ssd)
            if match_ssd(j)>0
                line([feature_points1(match_ssd(j),1) feature_points2(j,1)+cols1], ...
                     [feature_points1(match_ssd(j),2) feature_points2(j,2)], 'Color', 'c');
            end
        end
        for j = 1: length(match_correct)
            if match_correct(j)>0
                line([feature_points1(match_correct(j),1) feature_points2(j,1)+cols1], ...
                     [feature_points1(match_correct(j),2) feature_points2(j,2)], 'Color', 'b');
            end
        end
        hold off;
    end
end

%---------------------output data-----------------------------------------%
data(:,1)=repeatability;%m*n m: number of image, n: measure
data(:,2)=err_mean_out;
data(:,3)=variance_out;
data(:,4)=num_correct;
data(:,5)=num_total_match;
data(:,6)=correctness;
data(:,7)=num_fp_out1;
data(:,8)=num_fp_out2;
str_id_method = int2str(id_method);
name_data_save = [path_data '\data_method' str_id_method '.txt'];
if flag_overwrite==true
    dlmwrite(name_data_save, data);
end

name_points_repeatability = [path_data '\points_repeatability' str_id_method '.mat'];
if flag_overwrite==true
    save(name_points_repeatability, 'points_prj2_all', 'feature_points2_all', 'Homo_all', 'thresh_distance_all', 'imsz1');
end












